################################################################
#
#   Bell Runner - Bell
#
################################################################

from __future__ import division
import os
from pygame.transform import rotozoom
from albow.resource import resource_path, get_image, get_sound
from level import Item, get_item_image
from notes import NotesField, pitch_to_note, note_to_pitch

def get_bell_sounds():
	dir = resource_path("sounds/bells")
	names = os.listdir(dir)
	names.sort()
	sounds = []
	for name in names:
		if name.endswith(".ogg"):
			rsrc = "bells/%s" % name
			print "Loading bell sound", rsrc
			sound = get_sound(rsrc)
			sounds.append(sound)
	return sounds

rope_image = get_image("ropes/blue.png")
bell_sounds = get_bell_sounds()

#-----------------------------------------------------------------------------

class Bell(Item):

	editable = True
	resource_prefix = "bells"
	layer_name = 'bells'
	min_phase = -170
	max_phase = 170
	rotation_rate = 10 # degrees per frame

	pitch = 0
	phase = 0 # degrees
	initial_rope_length = 200
	actor_holding_rope = None
	rope_hold_offset = 0 # up from bottom end of rope
	going_up = False

	def __init__(self, rect, image_name):
		Item.__init__(self, rect, image_name)
		base = os.path.splitext(image_name)[0]
		self.pitch = int(base.split("-")[1])
	
	def reset(self):
		self.phase = 0
		self.rope_held_by(None, None)
	
	def create_edit_fields(self, fields):
		fields.append(('pitch', "Pitch", NotesField(20)))
	
	def load_edit_fields(self, fields):
		fields.pitch.set_text(pitch_to_note(self.pitch))
	
	def unload_edit_fields(self, fields):
		self.pitch = note_to_pitch(fields.pitch.get_text()[:1])

	def draw(self, surf):
		rect = self.rect
		base_image = get_item_image(self.__class__, self.image_name)
		image = rotozoom(base_image, -self.phase, 1)
		r = image.get_rect()
		r.center = self.rect.center
		surf.blit(image, r)
		rope_per_degree = 0.006 * rect.width
		rope_length = self.initial_rope_length + self.phase * rope_per_degree
		if rope_length > 0:
			s = rect.width / 128
			x = int(round(rect.centerx + 40 * s))
			y = rect.centery
			r = rope_image.get_rect()
			r.height = rope_length
			surf.blit(rope_image, (x, y), r)
	
	def begin_frame(self, level):
		actor = self.actor_holding_rope
		if actor:
			if self.going_up:
				if self.move_towards_phase(self.min_phase):
					self.going_up = False
			else:
				if self.move_towards_phase(self.max_phase):
					self.ring()
					level.bell_rung(self.pitch)
					self.going_up = True
		else:
			self.move_towards_phase(0)
	
	def move_towards_phase(self, target_phase):
		reached = False
		phase = self.phase
		if phase < target_phase:
			phase += self.rotation_rate
			reached = phase >= target_phase
		elif phase > target_phase:
			phase -= self.rotation_rate
			reached = phase <= target_phase
		else:
			reached = False
		if reached:
			phase = target_phase
		self.set_phase(phase)
		return reached
	
	def set_phase(self, phase):
		self.phase = phase
		actor = self.actor_holding_rope
		if actor:
			r = self.get_rope_rect()
			actor.set_grab_position((r.centerx, r.bottom - self.rope_hold_offset))
	
	def ring(self):
		bell_sounds[self.pitch].play()

	def get_rope_rect(self):
		p = self.get_rope_top_point()
		r = self.get_rope_source_rect()
		r.topleft = p
		return r
	
	def get_rope_top_point(self):
		rect = self.rect
		s = rect.width / 128
		x = int(round(rect.centerx + 40 * s))
		y = rect.centery
		return (x, y)
	
	def get_rope_source_rect(self):
		rect = self.rect
		rope_per_degree = 0.006 * rect.width
		rope_length = self.initial_rope_length + self.phase * rope_per_degree
		r = rope_image.get_rect()
		r.height = rope_length
		return r

	def rope_held_by(self, actor, grab_pt):
		self.actor_holding_rope = actor
		r = self.get_rope_rect()
		if grab_pt:
			self.rope_hold_offset = r.bottom - grab_pt[1]
		self.going_up = False
